@@ -14,7 +14,8 @@ class MyFileSystemEventHander(FileSystemEventHandler):
             if time.time()-eventgtimexz > 3:
                 eventgtimexz=time.time()
                 # print('* 更新文件：%s' % event.src_path)
-                time.sleep(2)
+                # time.sleep(2)
+                time.sleep(0.2)
                 self.restart()
 class Events:
     command = ['echo', 'ok']
