@@ -235,7 +235,7 @@ class cache:
         try:
             f=open(self.__config['path']+"/"+self.__name,"r")
         except Exception:
-            return None
+            return ""
         json_str=f.read()
         f.close()
         ar=json_decode(json_str)
@@ -244,7 +244,7 @@ class cache:
             if (times()-ar['time']) > ar['expire']:
                 
                 self.__delfilecache()
-                return None
+                return ""
             else:
                 return ar['values']
         else:
