@@ -2,12 +2,13 @@
 from .pymysql import connect,escape_string
 # import config.conf as config
 import kcweb.config as config
-import time,traceback,decimal,random
+import time,traceback,decimal,random,copy
 dbconfig=config.database
 class mysql:
     """数据库实例"""
     __config=dbconfig
     __conn={} #数据库链接对象
+    __connlists=[] #短连接列表  用于关闭
     __cursor=None #游标对象
     __errorcount=dbconfig['break'] #允许最大链接错误次数
     __errorcounts=0 #默认链接错误次数 
@@ -17,21 +18,44 @@ class mysql:
     __masteridentifier='' # 主服务器标识
     __slaveidentifier='' # 从服务器标识
     def __del__(self):
-        if not self.__config['pattern'] and self.__conn:
-            if isinstance(self.__conn,dict):
+        pass
+    def close(self):
+        "关闭连接,web一下模式下该方法无需理会，框架自动完成"
+        if not self.__config['pattern'] and mysql.__conn:
+            if self.__connlists:
+                for k in self.__connlists:
+                    try:
+                        mysql.__conn[k].close()
+                    except Exception as e:
+                        if self.__config['debug']:
+                            print("mysql短连接关闭失败",str(e),k)
+                    else:
+                        if self.__config['debug']:
+                            print("mysql短连接关闭成功",k)
+                mysql.__connlists=[]
+                mysql.__conn={}
+            elif isinstance(mysql.__conn,dict):
                 i=0
                 for thost in self.__config['host']:
                     identifier=thost+str(self.__config['port'][i])+self.__config['user'][i]+self.__config['password'][i]+self.__config['db'][i]
-                    for k in self.__conn[identifier]:
+                    # print(mysql.__conn)
+                    for k in mysql.__conn[identifier]:
                         try:
                             k['obj'].close()
+                            if self.__config['debug']:
+                                print("mysql短连接已关闭",k)
                         except Exception as e:
-                            print(k,"连接关闭失败",str(e))
+                            if self.__config['debug']:
+                                print("mysql短连接关闭失败",str(e),k)
                     i+=1
-                self.__conn={}
-            elif isinstance(self.__conn,object):
-                self.__conn.close()
-                self.__conn={}
+                mysql.__conn={}
+                self.__connlists=[]
+            elif isinstance(mysql.__conn,object):
+                mysql.__conn[self.__masteridentifier].close()
+                mysql.__conn={}
+                if self.__config['debug']:
+                    print("mysql短连接已关闭",mysql.__conn)
+        mysql.__config=copy.deepcopy(dbconfig)
     __dbcount=1
     def __setdbcount(self):
         "设置数据库配置总数量"
@@ -61,10 +85,10 @@ class mysql:
                     try:
                         k['obj'].close()
                         if self.__config['debug']:
-                            print(k,"关闭成功")
+                            print("mysql长连接关闭成功",k)
                     except Exception as e:
                         if self.__config['debug']:
-                            print(k,"连接关闭失败",str(e))
+                            print(k,"mysql长连接关闭失败",str(e))
                 mysql.__conn[identifier]=[]
     __dbobjident=None #集中式(单一服务器)并且长连接模式下随机服务器链接标识  和 分布式(主从服务器)模式下随机服务器链接标识
     def __connects(self,typess="DQL"):
@@ -88,7 +112,14 @@ class mysql:
                         if self.__config['debug']:
                             print("第%s次创建数据库链接对象，长连接模式" % (self.__errorcounts+1))
                 else:
-                    mysql.__conn=connect(host=self.__config['host'][0], port=self.__config['port'][0], user=self.__config['user'][0], password=self.__config['password'][0], db=self.__config['db'][0], charset=self.__config['charset'])
+                    self.__masteridentifier=self.__config['host'][0]+str(self.__config['port'][0])+self.__config['user'][0]+self.__config['password'][0]+self.__config['db'][0] # 服务器标识
+                    try:
+                        mysql.__conn[self.__masteridentifier]
+                    except KeyError: # 铺获未知异常
+                        mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][0], port=self.__config['port'][0], user=self.__config['user'][0], password=self.__config['password'][0], db=self.__config['db'][0], charset=self.__config['charset'])
+                        self.__connlists.append(self.__masteridentifier)
+                        if self.__config['debug']:
+                            print("mysql短连接已创建",self.__masteridentifier,mysql.__conn[self.__masteridentifier])
             elif self.__config['deploy']==1: # 分布式(主从服务器)
                 if self.__config['pattern']: # 长连接情况下
                     j=0
@@ -114,7 +145,7 @@ class mysql:
                             j=j+1
                         mysql.__conn[self.__masteridentifier]=masterlistsdb
                         if self.__config['debug']:
-                            print("次创建数据库链接对象，长连接模式（主）" % (self.__errorcounts+1))
+                            print("%d次创建数据库链接对象，长连接模式（主）" % (self.__errorcounts+1))
                     if self.__slaveidentifier not in mysql.__conn or len(mysql.__conn[self.__slaveidentifier])<self.__dbcount-self.__config['master_num']:
                         
                         j=self.__config['master_num']
@@ -129,43 +160,58 @@ class mysql:
                             j=j+1
                         mysql.__conn[self.__slaveidentifier]=slaveerlistsdb
                         if self.__config['debug']:
-                            print("创建数据库链接对象，长连接模式（从）" % (self.__errorcounts+1))
+                            print("%d创建数据库链接对象，长连接模式（从）" % (self.__errorcounts+1))
                 else:
                     if typess == "DQL": #数据查询语言DQL
                         if self.__config['master_dql']: #所有服务器随机
                             self.__dbobjident=random.randint(0,self.__dbcount-1)
-                            mysql.__conn=connect(host=self.__config['host'][self.__dbobjident], 
-                                port=self.__config['port'][self.__dbobjident], 
-                                user=self.__config['user'][self.__dbobjident], 
-                                password=self.__config['password'][self.__dbobjident], 
-                                db=self.__config['db'][self.__dbobjident], 
-                                charset=self.__config['charset']
-                            )
-                            if self.__config['debug']:
-                                print("创建所有数据库链接对象" % (self.__errorcounts+1)+self.__config['host'][self.__dbobjident])
+                            self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
+                            try:
+                                mysql.__conn[self.__masteridentifier]
+                            except KeyError:
+                                mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
+                                    port=self.__config['port'][self.__dbobjident], 
+                                    user=self.__config['user'][self.__dbobjident], 
+                                    password=self.__config['password'][self.__dbobjident], 
+                                    db=self.__config['db'][self.__dbobjident], 
+                                    charset=self.__config['charset']
+                                )
+                                self.__connlists.append(self.__masteridentifier)
+                                if self.__config['debug']:
+                                    print("创建所有数据库链接对象",(self.__errorcounts+1),self.__config['host'][self.__dbobjident])
                         else: #从服务器随机
                             self.__dbobjident=random.randint(self.__config['master_num'],self.__dbcount-1)
-                            mysql.__conn=connect(host=self.__config['host'][self.__dbobjident], 
-                                port=self.__config['port'][self.__dbobjident], 
-                                user=self.__config['user'][self.__dbobjident], 
-                                password=self.__config['password'][self.__dbobjident], 
-                                db=self.__config['db'][self.__dbobjident], 
-                                charset=self.__config['charset']
-                            )
-                            if self.__config['debug']:
-                                print("创建从数据库链接对象" % (self.__errorcounts+1)+self.__config['host'][self.__dbobjident]+str(self.__config['db'][self.__dbobjident]))
+                            self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
+                            try:
+                                mysql.__conn[self.__masteridentifier]
+                            except KeyError:
+                                mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
+                                    port=self.__config['port'][self.__dbobjident], 
+                                    user=self.__config['user'][self.__dbobjident], 
+                                    password=self.__config['password'][self.__dbobjident], 
+                                    db=self.__config['db'][self.__dbobjident], 
+                                    charset=self.__config['charset']
+                                )
+                                self.__connlists.append(self.__masteridentifier)
+                                if self.__config['debug']:
+                                    print("创建从数据库链接对象",str(self.__errorcounts+1),self.__config['host'][self.__dbobjident],str(self.__config['db'][self.__dbobjident]))
                     else:
                          #从服务器随机
                         self.__dbobjident=random.randint(self.__config['master_num'],self.__dbcount-1)
-                        mysql.__conn=connect(host=self.__config['host'][self.__dbobjident], 
-                            port=self.__config['port'][self.__dbobjident], 
-                            user=self.__config['user'][self.__dbobjident], 
-                            password=self.__config['password'][self.__dbobjident], 
-                            db=self.__config['db'][self.__dbobjident], 
-                            charset=self.__config['charset']
-                        )
-                        if self.__config['debug']:
-                            print("创建从数据库链接对象" % (self.__errorcounts+1)+self.__config['host'][self.__dbobjident]+str(self.__config['db'][self.__dbobjident]))
+                        self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
+                        try:
+                            mysql.__conn[self.__masteridentifier]
+                        except KeyError: 
+                            mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
+                                port=self.__config['port'][self.__dbobjident], 
+                                user=self.__config['user'][self.__dbobjident], 
+                                password=self.__config['password'][self.__dbobjident], 
+                                db=self.__config['db'][self.__dbobjident], 
+                                charset=self.__config['charset']
+                            )
+                            self.__connlists.append(self.__masteridentifier)
+                            if self.__config['debug']:
+                                print("创建从数据库链接对象" , (self.__errorcounts+1),self.__config['host'][self.__dbobjident],str(self.__config['db'][self.__dbobjident]))
                     
         except Exception as e: # 铺获未知异常
             try:
@@ -183,8 +229,8 @@ class mysql:
                             print("无法链接到数据库服务器，准备第%d次重新链接:%s" % (self.__errorcounts,e))
                         self.__connects(typess)
                     else:
+                        # self.__config=copy.deepcopy(dbconfig)
                         self.__errorcounts=0
-                        # self.__conn=None
                         raise Exception(e)
                 else:
                     self.__errorcounts=self.__errorcounts+1
@@ -193,11 +239,10 @@ class mysql:
                             print("无法链接到数据库服务器，开始重新链接")
                         self.__connects(typess)
                     else:
+                        # self.__config=copy.deepcopy(dbconfig)
                         self.__errorcounts=0
-                        # self.__conn=None
                         raise Exception(e)
             else:
-                # self.__conn=None
                 raise Exception(e)
         else:
             self.__errorcounts=0
@@ -278,7 +323,7 @@ class mysql:
                     self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例
                     res=self.__cursor.execute(self.__sql)
                 else:
-                    self.__cursor=self.__conn.cursor() #获取随机数据库链接实例
+                    self.__cursor=self.__conn[self.__masteridentifier].cursor() #获取随机数据库链接实例
                     res=self.__cursor.execute(self.__sql)
             elif self.__config['deploy']==1: # 分布式(主从服务器)
                 if self.__config['pattern']: # 长连接情况下
@@ -316,7 +361,7 @@ class mysql:
                     res=self.__cursor.execute(self.__sql)
                     self.__conn[bs][self.__dbobjident]['error']=0
                 else:
-                    self.__cursor=self.__conn.cursor()
+                    self.__cursor=self.__conn[self.__masteridentifier].cursor()
                     res=self.__cursor.execute(self.__sql)
         except Exception as e: # 铺获未知异常
             try:
@@ -364,11 +409,11 @@ class mysql:
                     if self.__config['debug']:
                         print("服务器正在被关闭，关闭当前连接后重试")
                     try:
-                        mysql.__conn.close() #关闭当前实例
+                        del mysql.__conn[self.__masteridentifier]
+                        mysql.__conn[self.__masteridentifier].close() #关闭当前实例
                     except Exception as e:
                         if self.__config['debug']:
                             print("关闭异常",e)
-                    # mysql.__conn=[] #父类数据库实例
                     self.__connects(typess)
                     self.__execute(typess)
             else:
@@ -389,6 +434,7 @@ class mysql:
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         if description is None:
             return res
         else:
@@ -416,6 +462,7 @@ class mysql:
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         if description is None:
             return res
         else:
@@ -450,6 +497,7 @@ class mysql:
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         lists=[]
         keys =[]
         for field in description:#获取字段
@@ -481,7 +529,7 @@ class mysql:
         description=self.__cursor.description #获取字段
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-
+        self.__None()
         item = dict()
         keys =[]
         for field in description:#获取字段
@@ -504,11 +552,19 @@ class mysql:
         self.__setsql('count')
         if self.__buildSql:
             self.__sqls="("+self.__sql+")"
+            self.__None()
             return self.__sql
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
-        cou=int(result[0][0])
+        if self.__group:
+            cou=len(result)
+        else:
+            try:
+                cou=int(result[0][0])
+            except IndexError:
+                cou=0
+        # self.__None()
         return cou
     def max(self,field):
         """查询某字段的最大值
@@ -519,11 +575,13 @@ class mysql:
         self.__setsql('max')
         if self.__buildSql:
             self.__sqls="("+self.__sql+")"
+            self.__None()
             return self.__sql
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
         cou=int(result[0][0])
+        self.__None()
         return cou
     def min(self,field):
         """查询某字段的最小值
@@ -534,10 +592,12 @@ class mysql:
         self.__setsql('min')
         if self.__buildSql:
             self.__sqls="("+self.__sql+")"
+            self.__None()
             return self.__sql
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         cou=int(result[0][0])
         return cou
     def avg(self,field):
@@ -549,10 +609,12 @@ class mysql:
         self.__setsql('avg')
         if self.__buildSql:
             self.__sqls="("+self.__sql+")"
+            self.__None()
             return self.__sql
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         cou=int(result[0][0])
         return cou
     def sum(self,field):
@@ -564,14 +626,35 @@ class mysql:
         self.__setsql('sum')
         if self.__buildSql:
             self.__sqls="("+self.__sql+")"
+            self.__None()
             return self.__sql
         self.__execute()
         result = self.__cursor.fetchall() #获取查询结果
         self.__cursor.close()
+        self.__None()
         cou=int(result[0][0])
         return cou
 
-    
+    def setinc(self,field,key=1,affair=False):
+        """更新字段增加
+         
+        参数 field 要更新的字段
+
+        参数 key 字段需要加多少
+
+        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
+        """
+        data={"field":field,"key":key}
+        self.__setsql('setinc',data)
+        res=self.__execute('DML')
+        if affair==False and self.__startTrans==False:
+            if not self.__config['pattern']:
+                self.__conn[self.__masteridentifier].commit()
+            else:
+                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
+        self.__cursor.close()
+        self.__None()
+        return res
     def update(self,data,affair=False):
         """数据表更新
          
@@ -583,10 +666,11 @@ class mysql:
         res=self.__execute('DML')
         if affair==False and self.__startTrans==False:
             if not self.__config['pattern']:
-                self.__conn.commit()
+                self.__conn[self.__masteridentifier].commit()
             else:
                 self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
         self.__cursor.close()
+        self.__None()
         return res
     def delete(self,affair=False):
         """数据表删除
@@ -600,10 +684,11 @@ class mysql:
             return 0
         if affair==False and self.__startTrans==False:
             if not self.__config['pattern']:
-                self.__conn.commit()
+                self.__conn[self.__masteridentifier].commit()
             else:
                 self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
         self.__cursor.close()
+        self.__None()
         return res
     def insert(self,dicts,affair=False):
         """插入数据库 单条插入或多条插入
@@ -618,10 +703,11 @@ class mysql:
         res=self.__execute('DML')
         if affair==False and self.__startTrans==False:
             if not self.__config['pattern']:
-                self.__conn.commit()
+                self.__conn[self.__masteridentifier].commit()
             else:
                 self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
         self.__cursor.close()
+        self.__None()
         return res
 
     __startTrans=False
@@ -633,20 +719,22 @@ class mysql:
 
         增删改后的任务进行提交
         """
-        if not self.__config['pattern']:
-            self.__conn.commit()
-        else:
-            self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
+        if self.__masteridentifier:
+            if not self.__config['pattern']:
+                self.__conn[self.__masteridentifier].commit()
+            else:
+                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
 
     def rollback(self):
         """事务回滚
 
         增删改后的任务进行撤销
         """
-        if not self.__config['pattern']:
-            self.__conn.rollback()
-        else:
-            self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].rollback()
+        if self.__masteridentifier:
+            if not self.__config['pattern']:
+                self.__conn[self.__masteridentifier].rollback()
+            else:
+                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].rollback()
     def getsql(self):
         """得到生成的sql语句"""
         return self.__sql
@@ -712,13 +800,18 @@ class mysql:
         self.__field=field
         return self
     __limit=[]
-    def limit(self,offset, length = None):
+    def limit(self,offset=1, length = None):
         """设置查询数量
 
         参数 offset：int 起始位置
 
         参数 length：int 查询数量
         """
+        if not offset:
+            offset=1
+        offset=int(offset)
+        if length:
+            length=int(length)
         self.__limit=[offset,length]
         return self
     def page(self,pagenow=1, length = 20):
@@ -728,6 +821,12 @@ class mysql:
 
         参数 length：int 查询数量
         """
+        if not pagenow:
+            pagenow=1
+        if not length:
+            length=20
+        pagenow=int(pagenow)
+        length=int(length)
         offset=(pagenow-1)*length
         self.__limit=[offset,length]
         return self
@@ -850,6 +949,8 @@ class mysql:
             self.__sql="SELECT AVG(%s) FROM %s" % (self.__field,self.__table)
         elif types=='sum':
             self.__sql="SELECT SUM(%s) FROM %s" % (self.__field,self.__table)
+        elif types=='setinc':
+            self.__sql="update %s set %s=%s+%s" % (self.__table,data['field'],data['field'],data['key'])
         elif types=='update':
             strs=''
             for k in data:
@@ -919,6 +1020,12 @@ class mysql:
                 self.__sql=self.__sql + " WHERE  %s" % self.__listTrans()
             else:
                 print("参数where类型错误")
+        if self.__group:
+            s=self.__group
+            if self.__group1:
+                for key in self.__group1:
+                    s=s+","+key
+            self.__sql=self.__sql+" GROUP BY "+s
         if self.__order:
             s=''
             if isinstance(self.__order,list):
@@ -952,12 +1059,6 @@ class mysql:
                 else:
                     s=self.__order
             self.__sql=self.__sql+" ORDER BY "+s
-        if self.__group:
-            s=self.__group
-            if self.__group1:
-                for key in self.__group1:
-                    s=s+","+key
-            self.__sql=self.__sql+" GROUP BY "+s
         if self.__having:
             self.__sql=self.__sql+" HAVING "+self.__having
         if self.__limit:
